@@ -13,9 +13,11 @@ from deform.widget import CheckedPasswordWidget
 from deform.widget import SequenceWidget
 from pyramid.exceptions import Forbidden
 from pyramid.httpexceptions import HTTPFound
+from pyramid.settings import asbool
 from pyramid.view import view_config
 from pyramid_deform import FormView
 
+from kotti import get_settings
 from kotti.events import UserDeleted
 from kotti.events import notify
 from kotti.message import email_set_password
@@ -70,7 +72,7 @@ def roles_form_handler(context, request, available_role_names, groups_lister):
             request.session.flash(
                 _(u'Your changes have been saved.'), 'success')
         else:
-            request.session.flash(_(u'No changes made.'), 'info')
+            request.session.flash(_(u'No changes were made.'), 'info')
 
     return changed
 
@@ -97,7 +99,7 @@ def search_principals(request, context=None, ignore=None, extra=()):
             if p.name not in ignore:
                 entries.append((p, list_groups_ext(p.name, context)))
         if not found:
-            flash(_(u'No users or groups found.'), 'info')
+            flash(_(u'No users or groups were found.'), 'info')
 
     return entries
 
@@ -268,8 +270,7 @@ def user_schema(base=PrincipalFull()):
     if has_password:
         schema['password'].description = _(
             u"Leave this empty and tick the 'Send password registration' "
-            "box below to have the user set their own password."
-            )
+            u"box below to have the user set their own password.")
     schema['title'].title = _(u"Full name")
     return schema
 
@@ -291,10 +292,11 @@ def _massage_groups_in(appstruct):
     The value in the form is 'name', not 'group:name', so we'll
     need to append that before we save.
     """
-    groups = appstruct['groups']
-    all_groups = list(appstruct['roles']) + [
+    groups = appstruct.get('groups', [])
+    all_groups = list(appstruct.get('roles', [])) + [
         u'group:%s' % g for g in groups if g]
-    del appstruct['roles']
+    if 'roles' in appstruct:
+        del appstruct['roles']
     appstruct['groups'] = all_groups
 
 
@@ -303,9 +305,9 @@ def _massage_groups_out(appstruct):
     split 'groups' into 'roles' and 'groups'.
     """
     d = appstruct
-    groups = [g.split(u'group:')[1] for g in d['groups']
+    groups = [g.split(u'group:')[1] for g in d.get('groups', u'')
               if g and g.startswith(u'group:')]
-    roles = [r for r in d['groups'] if r and r.startswith(u'role:')]
+    roles = [r for r in d.get('groups', u'') if r and r.startswith(u'role:')]
     d['groups'] = groups
     d['roles'] = roles
     return d
@@ -323,7 +325,7 @@ class UserAddFormView(AddFormView):
         schema.add(colander.SchemaNode(
             colander.Boolean(),
             name=u'send_email',
-            title=_(u'Send password registration link'),
+            title=_(u'Send password registration link.'),
             default=True,
             ))
         return schema
@@ -337,8 +339,9 @@ class UserAddFormView(AddFormView):
         get_principals()[name] = appstruct
         if send_email:
             email_set_password(get_principals()[name], self.request)
-        self.request.session.flash(_(u'${title} added.',
-            mapping=dict(title=appstruct['title'])), 'success')
+        self.request.session.flash(
+            _(u'${title} was added.',
+              mapping=dict(title=appstruct['title'])), 'success')
         location = self.request.url.split('?')[0] + '?' + urlencode(
             {'extra': name})
         return HTTPFound(location=location)
@@ -371,12 +374,10 @@ class UsersManage(FormView):
         self.request = request
 
     def __call__(self):
-        api = template_api(
-            self.context, self.request,
-            cp_links=CONTROL_PANEL_LINKS,
-            )
-        api.page_title = _(u"User Management - ${title}",
-                           mapping=dict(title=api.site_title))
+        api = template_api(self.context, self.request,
+                           cp_links=CONTROL_PANEL_LINKS)
+
+        api.page_title = _(u"User Management")
 
         principals = get_principals()
 
@@ -434,23 +435,29 @@ class UserManageFormView(UserEditFormView):
 
     buttons = (Button('save', _(u'Save')),
                Button('cancel', _(u'Cancel')),
-               Button('delete', _(u'Delete')))
+               Button('delete', _(u'Delete'), css_class='btn btn-danger'))
 
     def schema_factory(self):
         schema = user_schema()
         del schema['name']
-        del schema['password']
         return schema
 
     def before(self, form):
-        form.appstruct = _massage_groups_out(self.context.__dict__.copy())
+        context = self.context.__dict__.copy()
+        context['password'] = u''
+        form.appstruct = _massage_groups_out(context)
 
     def save_success(self, appstruct):
+        if appstruct.get('password'):
+            hashed = get_principals().hash_password(appstruct['password'])
+            appstruct['password'] = hashed
+        else:
+            appstruct.pop('password', None)
         _massage_groups_in(appstruct)
         return super(UserEditFormView, self).save_success(appstruct)
 
     def cancel_success(self, appstruct):
-        self.request.session.flash(_(u'No changes made.'), 'info')
+        self.request.session.flash(_(u'No changes were made.'), 'info')
         location = "%s/@@setup-users" % self.request.application_url
         return HTTPFound(location=location)
     cancel_failure = cancel_success
@@ -490,12 +497,11 @@ class UserManage(FormView):
 
         api = template_api(
             self.context, self.request,
-            page_title=_(u"Edit ${principal_type} - ${title}",
+            page_title=_(u"Edit ${principal_type} ${title}",
                          mapping=dict(principal_type=principal_type,
                                       title=self.context.title)),
             cp_links=CONTROL_PANEL_LINKS,
-            principal=principal,
-            )
+            principal=principal)
 
         form_view = self.GroupManageFormView if is_group \
             else self.UserManageFormView
@@ -519,7 +525,7 @@ def user_delete(context, request):
         user_or_group = request.params['name']
         principal = principals.search(name=user_or_group).first()
         if principal is None:
-            request.session.flash(_(u'User not found.'), 'error')
+            request.session.flash(_(u'User was not found.'), 'error')
         else:
             is_group = user_or_group.startswith("group:")
             principal_type = _(u"Group") if is_group else _(u"User")
@@ -528,23 +534,23 @@ def user_delete(context, request):
             if 'delete' in request.POST:
                 principals.__delitem__(principal.name)
                 notify(UserDeleted(principal, request))
-                request.session.flash(_(u'${principal_type} ${title} deleted.',
-                                        mapping=dict(principal_type=principal_type,
-                                                     title=principal.title)), 'info')
+                request.session.flash(
+                    _(u'${principal_type} ${title} was deleted.',
+                      mapping=dict(principal_type=principal_type,
+                                   title=principal.title)), 'info')
                 location = "%s/@@setup-users" % request.application_url
                 return HTTPFound(location=location)
 
             api = template_api(
                 context, request,
-                page_title=_(u"Delete ${principal_type} - ${title}",
+                page_title=_(u"Delete ${principal_type} ${title}",
                              mapping=dict(principal_type=principal_type,
                                           title=principal.title)),
                 principal_type=principal_type,
-                principal=principal,
-                )
+                principal=principal)
             return {'api': api, }
     else:
-        request.session.flash(_(u'No name given.'), 'error')
+        request.session.flash(_(u'No name was given.'), 'error')
 
     return {'api': template_api(context, request), }
 
